@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -84,10 +84,6 @@ void QDeclarativeCamera::_q_availabilityChanged(QMultimedia::AvailabilityStatus
     viewfinder you can use \l VideoOutput with the Camera set as the source.
 
     \qml
-
-    import QtQuick 2.0
-    import QtMultimedia 5.4
-
     Item {
         width: 640
         height: 360
@@ -778,7 +774,16 @@ void QDeclarativeCamera::setDigitalZoom(qreal value)
 /*!
     \qmlproperty variant QtMultimedia::Camera::mediaObject
 
-    This property holds the media object for the camera.
+    This property holds the native media object for the camera.
+
+    It can be used to get a pointer to a QCamera object in order to integrate with C++ code.
+
+    \code
+        QObject *qmlCamera; // The QML Camera object
+        QCamera *camera = qvariant_cast<QCamera *>(qmlCamera->property("mediaObject"));
+    \endcode
+
+    \note This property is not accessible from QML.
 */
 
 /*!
@@ -866,116 +871,42 @@ void QDeclarativeCamera::setDigitalZoom(qreal value)
 */
 
 /*!
+    \qmlpropertygroup QtMultimedia::Camera::metaData
     \qmlproperty variant QtMultimedia::Camera::metaData.cameraManufacturer
-
-    This property holds the name of the manufacturer of the camera.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.cameraModel
-
-    This property holds the name of the model of the camera.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.event
-
-    This property holds the event during which the photo or video is to be captured.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.subject
-
-    This property holds the name of the subject of the capture or recording.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.orientation
-
-    This property holds the clockwise rotation of the camera at time of capture.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.dateTimeOriginal
-
-    This property holds the initial time at which the photo or video is
-    captured.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsLatitude
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsLongitude
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsAltitude
-
-    These properties hold the geographic position in decimal degrees of the
-    camera at time of capture.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsTimestamp
-
-    This property holds the timestamp of the GPS position data.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsTrack
-
-    This property holds direction of movement of the camera at the time of
-    capture. It is measured in degrees clockwise from north.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsSpeed
-
-    This property holds the velocity in kilometers per hour of the camera at
-    time of capture.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsImgDirection
-
-    This property holds direction the camera is facing at the time of capture.
-    It is measured in degrees clockwise from north.
-
-    \sa {QMediaMetaData}
-    \since 5.4
-*/
-
-/*!
     \qmlproperty variant QtMultimedia::Camera::metaData.gpsProcessingMethod
 
-    This property holds the name of the method for determining the GPS position
-    data.
+    These properties hold the meta data for the camera captures.
+
+    \list
+    \li \c metaData.cameraManufacturer holds the name of the manufacturer of the camera.
+    \li \c metaData.cameraModel holds the name of the model of the camera.
+    \li \c metaData.event holds the event during which the photo or video is to be captured.
+    \li \c metaData.subject holds the name of the subject of the capture or recording.
+    \li \c metaData.orientation holds the clockwise rotation of the camera at time of capture.
+    \li \c metaData.dateTimeOriginal holds the initial time at which the photo or video is captured.
+    \li \c metaData.gpsLatitude holds the latitude of the camera in decimal degrees at time of capture.
+    \li \c metaData.gpsLongitude holds the longitude of the camera in decimal degrees at time of capture.
+    \li \c metaData.gpsAltitude holds the altitude of the camera in meters at time of capture.
+    \li \c metaData.gpsTimestamp holds the timestamp of the GPS position data.
+    \li \c metaData.gpsTrack holds direction of movement of the camera at the time of
+           capture. It is measured in degrees clockwise from north.
+    \li \c metaData.gpsSpeed holds the velocity in kilometers per hour of the camera at time of capture.
+    \li \c metaData.gpsImgDirection holds direction the camera is facing at the time of capture.
+           It is measured in degrees clockwise from north.
+    \li \c metaData.gpsProcessingMethod holds the name of the method for determining the GPS position.
+    \endlist
 
     \sa {QMediaMetaData}
     \since 5.4
@@ -989,20 +920,34 @@ QDeclarativeMediaMetaData *QDeclarativeCamera::metaData()
 }
 
 /*!
+    \qmlpropertygroup QtMultimedia::Camera::viewfinder
     \qmlproperty size QtMultimedia::Camera::viewfinder.resolution
-
-    This property holds the resolution of the camera viewfinder. If no
-    resolution is given the backend will use a default value.
-
-    \since 5.4
-*/
-
-/*!
     \qmlproperty real QtMultimedia::Camera::viewfinder.minimumFrameRate
     \qmlproperty real QtMultimedia::Camera::viewfinder.maximumFrameRate
 
-    These properties hold the limits of the preferred frame rate for the
-    viewfinder in frames per second.
+    These properties hold the viewfinder settings.
+
+    \c viewfinder.resolution holds the resolution of the camera viewfinder. If no
+    resolution is given or if it is empty, the backend uses a default value.
+
+    \c viewfinder.minimumFrameRate holds the minimum frame rate for the viewfinder in
+    frames per second. If no value is given or if set to \c 0, the backend uses a default value.
+
+    \c viewfinder.maximumFrameRate holds the maximum frame rate for the viewfinder in
+    frames per second. If no value is given or if set to \c 0, the backend uses a default value.
+
+    If \c viewfinder.minimumFrameRate is equal to \c viewfinder.maximumFrameRate, the frame rate is
+    fixed. If not, the actual frame rate fluctuates between the two values.
+
+    Changing the viewfinder settings while the camera is in the \c Camera.ActiveState state may
+    cause the camera to be restarted.
+
+    If the camera is used to capture videos or images, the viewfinder settings might be
+    ignored if they conflict with the capture settings. You can check the actual viewfinder settings
+    once the camera is in the \c Camera.ActiveStatus status.
+
+    Supported values can be retrieved with supportedViewfinderResolutions() and
+    supportedViewfinderFrameRateRanges().
 
     \since 5.4
  */
@@ -1015,6 +960,84 @@ QDeclarativeCameraViewfinder *QDeclarativeCamera::viewfinder()
     return m_viewfinder;
 }
 
+/*!
+    \qmlmethod list<size> QtMultimedia::Camera::supportedViewfinderResolutions(real minimumFrameRate = undefined, real maximumFrameRate = undefined)
+
+    Returns a list of supported viewfinder resolutions.
+
+    If both optional parameters \a minimumFrameRate and \a maximumFrameRate are specified, the
+    returned list is reduced to resolutions supported for the given frame rate range.
+
+    The camera must be loaded before calling this function, otherwise the returned list
+    is empty.
+
+    \sa {QtMultimedia::Camera::viewfinder}{viewfinder}
+
+    \since 5.5
+*/
+QJSValue QDeclarativeCamera::supportedViewfinderResolutions(qreal minimumFrameRate, qreal maximumFrameRate)
+{
+    QQmlEngine *engine = qmlEngine(this);
+
+    QCameraViewfinderSettings settings;
+    settings.setMinimumFrameRate(minimumFrameRate);
+    settings.setMaximumFrameRate(maximumFrameRate);
+    QList<QSize> resolutions = m_camera->supportedViewfinderResolutions(settings);
+
+    QJSValue supportedResolutions = engine->newArray(resolutions.count());
+    int i = 0;
+    Q_FOREACH (const QSize &resolution, resolutions) {
+        QJSValue size = engine->newObject();
+        size.setProperty(QStringLiteral("width"), resolution.width());
+        size.setProperty(QStringLiteral("height"), resolution.height());
+        supportedResolutions.setProperty(i++, size);
+    }
+
+    return supportedResolutions;
+}
+
+/*!
+    \qmlmethod list<object> QtMultimedia::Camera::supportedViewfinderFrameRateRanges(size resolution = undefined)
+
+    Returns a list of supported viewfinder frame rate ranges.
+
+    Each range object in the list has the \c minimumFrameRate and \c maximumFrameRate properties.
+
+    If the optional parameter \a resolution is specified, the returned list is reduced to frame rate
+    ranges supported for the given \a resolution.
+
+    The camera must be loaded before calling this function, otherwise the returned list
+    is empty.
+
+    \sa {QtMultimedia::Camera::viewfinder}{viewfinder}
+
+    \since 5.5
+*/
+QJSValue QDeclarativeCamera::supportedViewfinderFrameRateRanges(const QJSValue &resolution)
+{
+    QQmlEngine *engine = qmlEngine(this);
+
+    QCameraViewfinderSettings settings;
+    if (!resolution.isUndefined()) {
+        QJSValue width = resolution.property(QStringLiteral("width"));
+        QJSValue height = resolution.property(QStringLiteral("height"));
+        if (width.isNumber() && height.isNumber())
+            settings.setResolution(width.toInt(), height.toInt());
+    }
+    QList<QCamera::FrameRateRange> frameRateRanges = m_camera->supportedViewfinderFrameRateRanges(settings);
+
+    QJSValue supportedFrameRateRanges = engine->newArray(frameRateRanges.count());
+    int i = 0;
+    Q_FOREACH (const QCamera::FrameRateRange &frameRateRange, frameRateRanges) {
+        QJSValue range = engine->newObject();
+        range.setProperty(QStringLiteral("minimumFrameRate"), frameRateRange.minimumFrameRate);
+        range.setProperty(QStringLiteral("maximumFrameRate"), frameRateRange.maximumFrameRate);
+        supportedFrameRateRanges.setProperty(i++, range);
+    }
+
+    return supportedFrameRateRanges;
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qdeclarativecamera_p.cpp"
