@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -161,16 +153,16 @@ QAudioFormat QAudioOutput::format() const
 }
 
 /*!
-    Uses the \a device as the QIODevice to transfer data.
-    Passing a QIODevice allows the data to be transferred without any extra code.
-    All that is required is to open the QIODevice.
+    Starts transferring audio data from the \a device to the system's audio output.
+    The \a device must have been opened in the \l{QIODevice::ReadOnly}{ReadOnly} or
+    \l{QIODevice::ReadWrite}{ReadWrite} modes.
 
-    If able to successfully output audio data to the systems audio device the
-    state() is set to QAudio::ActiveState, error() is set to QAudio::NoError
+    If the QAudioOutput is able to successfully output audio data, state() returns
+    QAudio::ActiveState, error() returns QAudio::NoError
     and the stateChanged() signal is emitted.
 
-    If a problem occurs during this process the error() is set to QAudio::OpenError,
-    state() is set to QAudio::StoppedState and stateChanged() signal is emitted.
+    If a problem occurs during this process, error() returns QAudio::OpenError,
+    state() returns QAudio::StoppedState and the stateChanged() signal is emitted.
 
     \sa QIODevice
 */
@@ -180,15 +172,19 @@ void QAudioOutput::start(QIODevice* device)
 }
 
 /*!
-    Returns a pointer to the QIODevice being used to handle the data
-    transfer. This QIODevice can be used to write() audio data directly.
+    Returns a pointer to the internal QIODevice being used to transfer data to
+    the system's audio output. The device will already be open and
+    \l{QIODevice::write()}{write()} can write data directly to it.
 
-    If able to access the systems audio device the state() is set to
-    QAudio::IdleState, error() is set to QAudio::NoError
+    \note The pointer will become invalid after the stream is stopped or
+    if you start another stream.
+
+    If the QAudioOutput is able to access the system's audio device, state() returns
+    QAudio::IdleState, error() returns QAudio::NoError
     and the stateChanged() signal is emitted.
 
-    If a problem occurs during this process the error() is set to QAudio::OpenError,
-    state() is set to QAudio::StoppedState and stateChanged() signal is emitted.
+    If a problem occurs during this process, error() returns QAudio::OpenError,
+    state() returns QAudio::StoppedState and the stateChanged() signal is emitted.
 
     \sa QIODevice
 */
