@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -57,6 +49,7 @@ QT_BEGIN_NAMESPACE
 static void qRegisterCameraImageProcessingMetaTypes()
         {
             qRegisterMetaType<QCameraImageProcessing::WhiteBalanceMode>();
+            qRegisterMetaType<QCameraImageProcessing::ColorFilter>();
         }
 
 Q_CONSTRUCTOR_FUNCTION(qRegisterCameraImageProcessingMetaTypes)
@@ -325,5 +318,63 @@ void QCameraImageProcessing::setDenoisingLevel(qreal level)
     \value WhiteBalanceVendor       Base value for vendor defined white balance modes.
 */
 
+/*!
+    \enum QCameraImageProcessing::ColorFilter
+
+    \value ColorFilterNone               No filter is applied to images.
+    \value ColorFilterGrayscale          A grayscale filter.
+    \value ColorFilterNegative           A negative filter.
+    \value ColorFilterSolarize           A solarize filter.
+    \value ColorFilterSepia              A sepia filter.
+    \value ColorFilterPosterize          A posterize filter.
+    \value ColorFilterWhiteboard         A whiteboard filter.
+    \value ColorFilterBlackboard         A blackboard filter.
+    \value ColorFilterAqua               An aqua filter.
+    \value ColorFilterVendor             The base value for vendor defined filters.
+
+    \since 5.5
+*/
+
+/*!
+    Returns the color filter which will be applied to image data captured by the camera.
+
+    \since 5.5
+*/
+
+QCameraImageProcessing::ColorFilter QCameraImageProcessing::colorFilter() const
+{
+    return d_func()->imageControl->parameter(QCameraImageProcessingControl::ColorFilter)
+            .value<QCameraImageProcessing::ColorFilter>();
+}
+
+
+/*!
+    Sets the color \a filter which will be applied to image data captured by the camera.
+
+    \since 5.5
+*/
+
+void QCameraImageProcessing::setColorFilter(QCameraImageProcessing::ColorFilter filter)
+{
+    d_func()->imageControl->setParameter(
+                QCameraImageProcessingControl::ColorFilter,
+                QVariant::fromValue<QCameraImageProcessing::ColorFilter>(filter));
+}
+
+/*!
+    Returns true if a color \a filter is supported.
+
+    \since 5.5
+*/
+
+bool QCameraImageProcessing::isColorFilterSupported(QCameraImageProcessing::ColorFilter filter) const
+{
+    return d_func()->imageControl->isParameterValueSupported(
+                QCameraImageProcessingControl::ColorFilter,
+                QVariant::fromValue<QCameraImageProcessing::ColorFilter>(filter));
+
+}
+
+
 #include "moc_qcameraimageprocessing.cpp"
 QT_END_NAMESPACE
