@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -54,7 +54,7 @@ class QCameraImageProcessingPrivate;
 class Q_MULTIMEDIA_EXPORT QCameraImageProcessing : public QObject
 {
     Q_OBJECT
-    Q_ENUMS(WhiteBalanceMode)
+    Q_ENUMS(WhiteBalanceMode ColorFilter)
 public:
     enum WhiteBalanceMode {
         WhiteBalanceAuto = 0,
@@ -69,6 +69,19 @@ public:
         WhiteBalanceVendor = 1000
     };
 
+    enum ColorFilter {
+        ColorFilterNone,
+        ColorFilterGrayscale,
+        ColorFilterNegative,
+        ColorFilterSolarize,
+        ColorFilterSepia,
+        ColorFilterPosterize,
+        ColorFilterWhiteboard,
+        ColorFilterBlackboard,
+        ColorFilterAqua,
+        ColorFilterVendor = 1000
+    };
+
     bool isAvailable() const;
 
     WhiteBalanceMode whiteBalanceMode() const;
@@ -90,6 +103,10 @@ public:
     qreal denoisingLevel() const;
     void setDenoisingLevel(qreal value);
 
+    ColorFilter colorFilter() const;
+    void setColorFilter(ColorFilter filter);
+    bool isColorFilterSupported(ColorFilter filter) const;
+
 private:
     friend class QCamera;
     friend class QCameraPrivate;
@@ -104,7 +121,9 @@ private:
 QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QCameraImageProcessing::WhiteBalanceMode)
+Q_DECLARE_METATYPE(QCameraImageProcessing::ColorFilter)
 
 Q_MEDIA_ENUM_DEBUG(QCameraImageProcessing, WhiteBalanceMode)
+Q_MEDIA_ENUM_DEBUG(QCameraImageProcessing, ColorFilter)
 
 #endif  // QCAMERAIMAGEPROCESSING_H
