@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -214,9 +206,10 @@ bool QAndroidCameraSession::open()
 
     if (m_camera) {
         connect(m_camera, SIGNAL(pictureExposed()), this, SLOT(onCameraPictureExposed()));
-        connect(m_camera, SIGNAL(previewFetched(QByteArray)), this, SLOT(onCameraPreviewFetched(QByteArray)));
-        connect(m_camera, SIGNAL(frameFetched(QByteArray)),
-                this, SLOT(onCameraFrameFetched(QByteArray)),
+        connect(m_camera, SIGNAL(lastPreviewFrameFetched(QByteArray,int,int)),
+                this, SLOT(onLastPreviewFrameFetched(QByteArray,int,int)));
+        connect(m_camera, SIGNAL(newPreviewFrame(QByteArray,int,int)),
+                this, SLOT(onNewPreviewFrame(QByteArray,int,int)),
                 Qt::DirectConnection);
         connect(m_camera, SIGNAL(pictureCaptured(QByteArray)), this, SLOT(onCameraPictureCaptured(QByteArray)));
         connect(m_camera, SIGNAL(previewStarted()), this, SLOT(onCameraPreviewStarted()));
@@ -229,7 +222,7 @@ bool QAndroidCameraSession::open()
         if (m_camera->getPreviewFormat() != AndroidCamera::NV21)
             m_camera->setPreviewFormat(AndroidCamera::NV21);
 
-        m_camera->fetchEachFrame(m_videoProbes.count());
+        m_camera->notifyNewFrames(m_videoProbes.count());
 
         emit opened();
     } else {
@@ -284,33 +277,40 @@ void QAndroidCameraSession::adjustViewfinderSize(const QSize &captureSize, bool
     if (!m_camera)
         return;
 
-    QSize viewfinderResolution = m_camera->previewSize();
-    const qreal aspectRatio = qreal(captureSize.width()) / qreal(captureSize.height());
-    if (viewfinderResolution.isValid() &&
-            qFuzzyCompare(aspectRatio,
-                          qreal(viewfinderResolution.width()) / viewfinderResolution.height())) {
-        return;
-    }
+    QSize currentViewfinderResolution = m_camera->previewSize();
+    QSize adjustedViewfinderResolution;
 
-    QList<QSize> previewSizes = m_camera->getSupportedPreviewSizes();
-    for (int i = previewSizes.count() - 1; i >= 0; --i) {
-        const QSize &size = previewSizes.at(i);
+    if (m_captureMode.testFlag(QCamera::CaptureVideo) && m_camera->getPreferredPreviewSizeForVideo().isEmpty()) {
+        // According to the Android doc, if getPreferredPreviewSizeForVideo() returns null, it means
+        // the preview size cannot be different from the capture size
+        adjustedViewfinderResolution = captureSize;
+    } else {
         // search for viewfinder resolution with the same aspect ratio
-        if (qFuzzyCompare(aspectRatio, (static_cast<qreal>(size.width())/static_cast<qreal>(size.height())))) {
-            viewfinderResolution = size;
-            break;
+        const qreal aspectRatio = qreal(captureSize.width()) / qreal(captureSize.height());
+        QList<QSize> previewSizes = m_camera->getSupportedPreviewSizes();
+        for (int i = previewSizes.count() - 1; i >= 0; --i) {
+            const QSize &size = previewSizes.at(i);
+            if (qAbs(aspectRatio - (qreal(size.width()) / size.height())) < 0.01) {
+                adjustedViewfinderResolution = size;
+                break;
+            }
+        }
+
+        if (!adjustedViewfinderResolution.isValid()) {
+            qWarning("Cannot find a viewfinder resolution matching the capture aspect ratio.");
+            return;
         }
     }
 
-    if (m_camera->previewSize() != viewfinderResolution) {
+    if (currentViewfinderResolution != adjustedViewfinderResolution) {
         if (m_videoOutput)
-            m_videoOutput->setVideoSize(viewfinderResolution);
+            m_videoOutput->setVideoSize(adjustedViewfinderResolution);
 
         // if preview is started, we have to stop it first before changing its size
         if (m_previewStarted && restartPreview)
             m_camera->stopPreview();
 
-        m_camera->setPreviewSize(viewfinderResolution);
+        m_camera->setPreviewSize(adjustedViewfinderResolution);
 
         // restart preview
         if (m_previewStarted && restartPreview)
@@ -411,7 +411,7 @@ void QAndroidCameraSession::addProbe(QAndroidMediaVideoProbeControl *probe)
     if (probe)
         m_videoProbes << probe;
     if (m_camera)
-        m_camera->fetchEachFrame(m_videoProbes.count());
+        m_camera->notifyNewFrames(m_videoProbes.count());
     m_videoProbesMutex.unlock();
 }
 
@@ -420,7 +420,7 @@ void QAndroidCameraSession::removeProbe(QAndroidMediaVideoProbeControl *probe)
     m_videoProbesMutex.lock();
     m_videoProbes.remove(probe);
     if (m_camera)
-        m_camera->fetchEachFrame(m_videoProbes.count());
+        m_camera->notifyNewFrames(m_videoProbes.count());
     m_videoProbesMutex.unlock();
 }
 
@@ -563,25 +563,54 @@ void QAndroidCameraSession::onCameraPictureExposed()
     m_camera->fetchLastPreviewFrame();
 }
 
-void QAndroidCameraSession::onCameraPreviewFetched(const QByteArray &preview)
+void QAndroidCameraSession::onLastPreviewFrameFetched(const QByteArray &preview, int width, int height)
 {
     if (preview.size()) {
         QtConcurrent::run(this, &QAndroidCameraSession::processPreviewImage,
                           m_currentImageCaptureId,
                           preview,
+                          width,
+                          height,
                           m_camera->getRotation());
     }
 }
 
-void QAndroidCameraSession::onCameraFrameFetched(const QByteArray &frame)
+void QAndroidCameraSession::processPreviewImage(int id, const QByteArray &data, int width, int height, int rotation)
+{
+    emit imageCaptured(id, prepareImageFromPreviewData(data, width, height, rotation));
+}
+
+QImage QAndroidCameraSession::prepareImageFromPreviewData(const QByteArray &data, int width, int height, int rotation)
+{
+    QImage result(width, height, QImage::Format_ARGB32);
+    qt_convert_NV21_to_ARGB32((const uchar *)data.constData(),
+                              (quint32 *)result.bits(),
+                              width,
+                              height);
+
+    QTransform transform;
+
+    // Preview display of front-facing cameras is flipped horizontally, but the frame data
+    // we get here is not. Flip it ourselves if the camera is front-facing to match what the user
+    // sees on the viewfinder.
+    if (m_camera->getFacing() == AndroidCamera::CameraFacingFront)
+        transform.scale(-1, 1);
+
+    transform.rotate(rotation);
+
+    result = result.transformed(transform);
+
+    return result;
+}
+
+void QAndroidCameraSession::onNewPreviewFrame(const QByteArray &frame, int width, int height)
 {
     m_videoProbesMutex.lock();
     if (frame.size() && m_videoProbes.count()) {
-        const QSize frameSize = m_camera->previewSize();
         // Bytes per line should be only for the first plane. For NV21, the Y plane has 8 bits
         // per sample, so bpl == width
-        QVideoFrame videoFrame(new DataVideoBuffer(frame, frameSize.width()),
-                               frameSize,
+        QVideoFrame videoFrame(new DataVideoBuffer(frame, width),
+                               QSize(width, height),
                                QVideoFrame::Format_NV21);
         foreach (QAndroidMediaVideoProbeControl *probe, m_videoProbes)
             probe->newFrameProbed(videoFrame);
@@ -667,35 +696,6 @@ void QAndroidCameraSession::processCapturedImage(int id,
     }
 }
 
-void QAndroidCameraSession::processPreviewImage(int id, const QByteArray &data, int rotation)
-{
-    emit imageCaptured(id, prepareImageFromPreviewData(data, rotation));
-}
-
-QImage QAndroidCameraSession::prepareImageFromPreviewData(const QByteArray &data, int rotation)
-{
-    QSize frameSize = m_camera->previewSize();
-    QImage result(frameSize, QImage::Format_ARGB32);
-    qt_convert_NV21_to_ARGB32((const uchar *)data.constData(),
-                              (quint32 *)result.bits(),
-                              frameSize.width(),
-                              frameSize.height());
-
-    QTransform transform;
-
-    // Preview display of front-facing cameras is flipped horizontally, but the frame data
-    // we get here is not. Flip it ourselves if the camera is front-facing to match what the user
-    // sees on the viewfinder.
-    if (m_camera->getFacing() == AndroidCamera::CameraFacingFront)
-        transform.scale(-1, 1);
-
-    transform.rotate(rotation);
-
-    result = result.transformed(transform);
-
-    return result;
-}
-
 void QAndroidCameraSession::onVideoOutputReady(bool ready)
 {
     if (ready && m_state == QCamera::ActiveState)
