@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -45,6 +37,36 @@
 
 QT_BEGIN_NAMESPACE
 
+class StateChangeNotifier
+{
+public:
+    StateChangeNotifier(QAndroidMediaPlayerControl *mp)
+        : mControl(mp)
+        , mPreviousState(mp->state())
+        , mPreviousMediaStatus(mp->mediaStatus())
+    {
+        ++mControl->mActiveStateChangeNotifiers;
+    }
+
+    ~StateChangeNotifier()
+    {
+        if (--mControl->mActiveStateChangeNotifiers)
+            return;
+
+        if (mPreviousState != mControl->state())
+            Q_EMIT mControl->stateChanged(mControl->state());
+
+        if (mPreviousMediaStatus != mControl->mediaStatus())
+            Q_EMIT mControl->mediaStatusChanged(mControl->mediaStatus());
+    }
+
+private:
+    QAndroidMediaPlayerControl *mControl;
+    QMediaPlayer::State mPreviousState;
+    QMediaPlayer::MediaStatus mPreviousMediaStatus;
+};
+
+
 QAndroidMediaPlayerControl::QAndroidMediaPlayerControl(QObject *parent)
     : QMediaPlayerControl(parent),
       mMediaPlayer(new AndroidMediaPlayer),
@@ -63,7 +85,9 @@ QAndroidMediaPlayerControl::QAndroidMediaPlayerControl(QObject *parent)
       mPendingPosition(-1),
       mPendingSetMedia(false),
       mPendingVolume(-1),
-      mPendingMute(-1)
+      mPendingMute(-1),
+      mReloadingMedia(false),
+      mActiveStateChangeNotifiers(0)
 {
     connect(mMediaPlayer,SIGNAL(bufferingChanged(qint32)),
             this,SLOT(onBufferingChanged(qint32)));
@@ -115,17 +139,14 @@ qint64 QAndroidMediaPlayerControl::position() const
     if (mCurrentMediaStatus == QMediaPlayer::EndOfMedia)
         return duration();
 
-    if ((mState & (AndroidMediaPlayer::Idle
-                   | AndroidMediaPlayer::Initialized
-                   | AndroidMediaPlayer::Prepared
+    if ((mState & (AndroidMediaPlayer::Prepared
                    | AndroidMediaPlayer::Started
                    | AndroidMediaPlayer::Paused
-                   | AndroidMediaPlayer::Stopped
-                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
-         return (mPendingPosition == -1) ? 0 : mPendingPosition;
+                   | AndroidMediaPlayer::PlaybackCompleted))) {
+        return mMediaPlayer->getCurrentPosition();
     }
 
-    return (mCurrentState == QMediaPlayer::StoppedState) ? 0 : mMediaPlayer->getCurrentPosition();
+    return (mPendingPosition == -1) ? 0 : mPendingPosition;
 }
 
 void QAndroidMediaPlayerControl::setPosition(qint64 position)
@@ -135,24 +156,25 @@ void QAndroidMediaPlayerControl::setPosition(qint64 position)
 
     const int seekPosition = (position > INT_MAX) ? INT_MAX : position;
 
-    if ((mState & (AndroidMediaPlayer::Prepared
-                   | AndroidMediaPlayer::Started
-                   | AndroidMediaPlayer::Paused
-                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
-        if (mPendingPosition != seekPosition) {
-            mPendingPosition = seekPosition;
-            Q_EMIT positionChanged(seekPosition);
-        }
+    if (seekPosition == this->position())
         return;
-    }
+
+    StateChangeNotifier notifier(this);
 
     if (mCurrentMediaStatus == QMediaPlayer::EndOfMedia)
         setMediaStatus(QMediaPlayer::LoadedMedia);
 
-    mMediaPlayer->seekTo(seekPosition);
+    if ((mState & (AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        mPendingPosition = seekPosition;
+    } else {
+        mMediaPlayer->seekTo(seekPosition);
 
-    if (mPendingPosition != -1) {
-        mPendingPosition = -1;
+        if (mPendingPosition != -1) {
+            mPendingPosition = -1;
+        }
     }
 
     Q_EMIT positionChanged(seekPosition);
@@ -283,9 +305,11 @@ const QIODevice *QAndroidMediaPlayerControl::mediaStream() const
 void QAndroidMediaPlayerControl::setMedia(const QMediaContent &mediaContent,
                                           QIODevice *stream)
 {
-    const bool reloading = (mMediaContent == mediaContent);
+    StateChangeNotifier notifier(this);
 
-    if (!reloading) {
+    mReloadingMedia = (mMediaContent == mediaContent) && !mPendingSetMedia;
+
+    if (!mReloadingMedia) {
         mMediaContent = mediaContent;
         mMediaStream = stream;
     }
@@ -296,41 +320,29 @@ void QAndroidMediaPlayerControl::setMedia(const QMediaContent &mediaContent,
 
     if (mediaContent.isNull()) {
         setMediaStatus(QMediaPlayer::NoMedia);
-        return;
-    }
-
-    if (mVideoOutput && !mVideoOutput->isReady()) {
-        // if a video output is set but the video texture is not ready, delay loading the media
-        // since it can cause problems on some hardware
-        mPendingSetMedia = true;
-        return;
-    }
-
-    const QUrl url = mediaContent.canonicalUrl();
-    QString mediaPath;
-    if (url.scheme() == QLatin1String("qrc")) {
-        const QString path = url.toString().mid(3);
-        mTempFile.reset(QTemporaryFile::createNativeFile(path));
-        if (!mTempFile.isNull())
-            mediaPath = QStringLiteral("file://") + mTempFile->fileName();
     } else {
-        mediaPath = url.toString();
+        if (mVideoOutput && !mVideoOutput->isReady()) {
+            // if a video output is set but the video texture is not ready, delay loading the media
+            // since it can cause problems on some hardware
+            mPendingSetMedia = true;
+            return;
+        }
+
+        if (mVideoSize.isValid() && mVideoOutput)
+            mVideoOutput->setVideoSize(mVideoSize);
+
+        if ((mMediaPlayer->display() == 0) && mVideoOutput)
+            mMediaPlayer->setDisplay(mVideoOutput->surfaceTexture());
+        mMediaPlayer->setDataSource(mediaContent.canonicalUrl().toString(QUrl::FullyEncoded));
+        mMediaPlayer->prepareAsync();
     }
 
-    if (mVideoSize.isValid() && mVideoOutput)
-        mVideoOutput->setVideoSize(mVideoSize);
-
-    if ((mMediaPlayer->display() == 0) && mVideoOutput)
-        mMediaPlayer->setDisplay(mVideoOutput->surfaceTexture());
-    mMediaPlayer->setDataSource(mediaPath);
-    mMediaPlayer->prepareAsync();
-
-    if (!reloading) {
+    if (!mReloadingMedia)
         Q_EMIT mediaChanged(mMediaContent);
-        Q_EMIT actualMediaLocationChanged(mediaPath);
-    }
 
     resetBufferingProgress();
+
+    mReloadingMedia = false;
 }
 
 void QAndroidMediaPlayerControl::setVideoOutput(QObject *videoOutput)
@@ -354,6 +366,8 @@ void QAndroidMediaPlayerControl::setVideoOutput(QObject *videoOutput)
 
 void QAndroidMediaPlayerControl::play()
 {
+    StateChangeNotifier notifier(this);
+
     // We need to prepare the mediaplayer again.
     if ((mState & AndroidMediaPlayer::Stopped) && !mMediaContent.isNull()) {
         setMedia(mMediaContent, mMediaStream);
@@ -374,6 +388,8 @@ void QAndroidMediaPlayerControl::play()
 
 void QAndroidMediaPlayerControl::pause()
 {
+    StateChangeNotifier notifier(this);
+
     setState(QMediaPlayer::PausedState);
 
     if ((mState & (AndroidMediaPlayer::Started
@@ -388,6 +404,8 @@ void QAndroidMediaPlayerControl::pause()
 
 void QAndroidMediaPlayerControl::stop()
 {
+    StateChangeNotifier notifier(this);
+
     setState(QMediaPlayer::StoppedState);
 
     if ((mState & (AndroidMediaPlayer::Prepared
@@ -405,6 +423,8 @@ void QAndroidMediaPlayerControl::stop()
 
 void QAndroidMediaPlayerControl::onInfo(qint32 what, qint32 extra)
 {
+    StateChangeNotifier notifier(this);
+
     Q_UNUSED(extra);
     switch (what) {
     case AndroidMediaPlayer::MEDIA_INFO_UNKNOWN:
@@ -436,6 +456,8 @@ void QAndroidMediaPlayerControl::onInfo(qint32 what, qint32 extra)
 
 void QAndroidMediaPlayerControl::onError(qint32 what, qint32 extra)
 {
+    StateChangeNotifier notifier(this);
+
     QString errorString;
     QMediaPlayer::Error error = QMediaPlayer::ResourceError;
 
@@ -488,6 +510,8 @@ void QAndroidMediaPlayerControl::onError(qint32 what, qint32 extra)
 
 void QAndroidMediaPlayerControl::onBufferingChanged(qint32 percent)
 {
+    StateChangeNotifier notifier(this);
+
     mBuffering = percent != 100;
     mBufferPercent = percent;
 
@@ -519,6 +543,8 @@ void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
         return;
     }
 
+    StateChangeNotifier notifier(this);
+
     mState = state;
     switch (mState) {
     case AndroidMediaPlayer::Idle:
@@ -526,7 +552,8 @@ void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
     case AndroidMediaPlayer::Initialized:
         break;
     case AndroidMediaPlayer::Preparing:
-        setMediaStatus(QMediaPlayer::LoadingMedia);
+        if (!mReloadingMedia)
+            setMediaStatus(QMediaPlayer::LoadingMedia);
         break;
     case AndroidMediaPlayer::Prepared:
         setMediaStatus(QMediaPlayer::LoadedMedia);
@@ -547,6 +574,7 @@ void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
         } else {
             setMediaStatus(QMediaPlayer::BufferedMedia);
         }
+        Q_EMIT positionChanged(position());
         break;
     case AndroidMediaPlayer::Paused:
         setState(QMediaPlayer::PausedState);
@@ -555,27 +583,32 @@ void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
         setState(QMediaPlayer::StoppedState);
         setMediaStatus(QMediaPlayer::UnknownMediaStatus);
         mMediaPlayer->release();
+        Q_EMIT positionChanged(0);
         break;
     case AndroidMediaPlayer::Stopped:
         setState(QMediaPlayer::StoppedState);
         setMediaStatus(QMediaPlayer::LoadedMedia);
-        setPosition(0);
+        Q_EMIT positionChanged(0);
         break;
     case AndroidMediaPlayer::PlaybackCompleted:
         setState(QMediaPlayer::StoppedState);
-        setPosition(0);
         setMediaStatus(QMediaPlayer::EndOfMedia);
         break;
     case AndroidMediaPlayer::Uninitialized:
-        // reset some properties
-        resetBufferingProgress();
-        mPendingPosition = -1;
-        mPendingSetMedia = false;
-        mPendingState = -1;
+        // reset some properties (unless we reload the same media)
+        if (!mReloadingMedia) {
+            resetBufferingProgress();
+            mPendingPosition = -1;
+            mPendingSetMedia = false;
+            mPendingState = -1;
 
-        setAudioAvailable(false);
-        setVideoAvailable(false);
-        setSeekable(true);
+            Q_EMIT durationChanged(0);
+            Q_EMIT positionChanged(0);
+
+            setAudioAvailable(false);
+            setVideoAvailable(false);
+            setSeekable(true);
+        }
         break;
     default:
         break;
@@ -607,7 +640,6 @@ void QAndroidMediaPlayerControl::setState(QMediaPlayer::State state)
         return;
 
     mCurrentState = state;
-    Q_EMIT stateChanged(mCurrentState);
 }
 
 void QAndroidMediaPlayerControl::setMediaStatus(QMediaPlayer::MediaStatus status)
@@ -615,14 +647,13 @@ void QAndroidMediaPlayerControl::setMediaStatus(QMediaPlayer::MediaStatus status
     if (mCurrentMediaStatus == status)
         return;
 
+    mCurrentMediaStatus = status;
+
     if (status == QMediaPlayer::NoMedia || status == QMediaPlayer::InvalidMedia)
         Q_EMIT durationChanged(0);
 
     if (status == QMediaPlayer::EndOfMedia)
-        Q_EMIT durationChanged(duration());
-
-    mCurrentMediaStatus = status;
-    Q_EMIT mediaStatusChanged(mCurrentMediaStatus);
+        Q_EMIT positionChanged(position());
 
     updateBufferStatus();
 }
@@ -667,8 +698,8 @@ void QAndroidMediaPlayerControl::resetBufferingProgress()
 void QAndroidMediaPlayerControl::flushPendingStates()
 {
     if (mPendingSetMedia) {
-        mPendingSetMedia = false;
         setMedia(mMediaContent, 0);
+        mPendingSetMedia = false;
         return;
     }
 
