@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -44,6 +44,8 @@
 
 
 #include <QtCore/qt_windows.h>
+#include <QtCore/QDataStream>
+#include <mmsystem.h>
 #include "qwindowsaudiodeviceinfo.h"
 #include "qwindowsaudioutils.h"
 
@@ -214,10 +216,10 @@ bool QWindowsAudioDeviceInfo::testSettings(const QAudioFormat& format) const
     if (qt_convertFormat(format, &wfx)) {
         // query only, do not open device
         if (mode == QAudio::AudioOutput) {
-            return (waveOutOpen(NULL, UINT_PTR(devId), &wfx.Format, NULL, NULL,
+            return (waveOutOpen(NULL, UINT_PTR(devId), &wfx.Format, 0, 0,
                                 WAVE_FORMAT_QUERY) == MMSYSERR_NOERROR);
         } else { // AudioInput
-            return (waveInOpen(NULL, UINT_PTR(devId), &wfx.Format, NULL, NULL,
+            return (waveInOpen(NULL, UINT_PTR(devId), &wfx.Format, 0, 0,
                                 WAVE_FORMAT_QUERY) == MMSYSERR_NOERROR);
         }
     }
